/* fileviewer.p
 *
 */

define input parameter fileName as character no-undo.

/* define variable fileName as character no-undo.	*/
/* fileName = session:parameter.			*/

define variable e  as handle    no-undo.			/* editor widget	*/
define variable f  as handle    no-undo.			/* frame		*/
define variable w  as integer   no-undo.			/* screen width		*/
define variable s  as character no-undo.			/* search string	*/

if session:batch = yes then return.

create widget-pool.

file-info:file-name = fileName.
if file-info:full-pathname = ? then return.

w = current-window:width-chars.			/* this results in line wrap on narrow screens					*/

w = 255.					/* this mostly prevents line wrap on narrow screens				*/

create frame f.
assign
  f:row          = 1
  f:column       = 1
  f:height-chars = screen-lines
  f:width-chars  = w 				/* w * 2 */
  f:box          = no
  f:top-only     = false
  f:overlay      = true
  f:scrollable   = false
  f:name         = "fileViewer"
  f:visible      = no 				/* yes */
  f:hidden       = yes 				/* no */
.

create editor e.
assign
  e:frame        = f
  e:inner-chars  = w - 2 			/* f:width-chars * 2 */
  e:inner-lines  = f:height-chars
  e:box          = no
  e:word-wrap    = no
  e:scrollbar-horizontal = false
  e:scrollbar-vertical   = false
  e:pfcolor      = 0				/* tty prompt color:  0 = normal, 1 = underline, 2 = reverse, 3 = bold		*/
.

if session:window-system <> "tty" then
  e:large = true.

assign
  e:visible      = yes
  e:hidden       = no
  e:sensitive    = yes
.

/*
 * if session:window-system = "tty" then
 *   e:pfcolor      = 0.			/* 0 = normal, 1 = underline, 2 = reverse, 3 = bold		*/
 */

e:read-file( file-info:full-pathname ) no-error.
e:read-only = true.


on "^" of e do:
  e:cursor-char = 1 no-error.
  return.
end.

on "$" of e do:					/* handling of the last line in the file is a bit wonky but not currently worth dealing with IMHO	*/
  e:cursor-line = e:cursor-line + 1 no-error.
  e:cursor-char = 1 no-error.
  e:cursor-offset = e:cursor-offset - 1 no-error.
  return.
end.

on "h" of e do:
  e:cursor-offset = e:cursor-offset - 1 no-error.
  return.
end.

on "j" of e do:
  e:cursor-line = e:cursor-line + 1 no-error.
  return.
end.

on "k" of e do:
  e:cursor-line = e:cursor-line - 1 no-error.
  return.
end.

on "l" of e do:
  e:cursor-offset = e:cursor-offset + 1 no-error.
  return.
end.

on "+", "d" of e do:
  e:cursor-line = e:cursor-line + screen-lines no-error.
  return.
end.

on "-", "u" of e do:
  e:cursor-line = max( 1, e:cursor-line - screen-lines ) no-error.
  return.
end.

on "<" of e do:
  e:cursor-line = 1.
  return.
end.

on ">" of e do:
  e:move-to-eof().
  return.
end.

/* search() flags bitmap:
 *
 *   1	FIND-NEXT-OCCURRENCE
 *   2	FIND-PREV-OCCURRENCE
 *   4	FIND-CASE-SENSITIVE
 *  16	FIND-WRAP-AROUND
 *  32	FIND-SELECT (does nothing for chui but we will use it anyway just in case that changes)
 *
 */

on "/", "f", "s" of e do:
  message "Search for:" update s format "x(30)".
  run statusLine.
  e:search( s, 17 + 32 ).
  return.
end.

on "p" of e do:
  e:search( s, 18 + 32 ).
  return.
end.

on "n" of e do:
  e:search( s, 17 + 32 ).
  return.
end.

on "?" of e do:
  publish "fileViewerHelp".
  return.
end.


apply "entry" to e.
run statusLine.

wait-for "f1", "go", "f4", "end-error", "q", "x" of e.

put screen row screen-lines + 1 column 1 fill( " ", w ).
put screen row screen-lines + 2 column 1 fill( " ", w ).

assign
  e:sensitive   = no
  e:visible     = no
  e:hidden      = yes
  f:visible     = no
  f:hidden      = yes
 no-error.

delete object e no-error.
delete object f no-error.

return.


procedure statusLine:

  define variable d as character no-undo.

  d = substitute( "==== ProTop File Viewer == File: &1 ==== Lines: &2 ==== Bytes: &3 ====", fileName, e:num-lines, e:length ).

  d = d + fill( "=", w - ( length( d ) + 20 )) + "==== ? for help ====".

  put screen color value( "blue/gray" ) row screen-lines + 1 column 1 d.

              /* "===============================================================x================" (80) */
  status default "? = help   q = quit   $ = EOL   ^ = BOL   > = EOF   < = BOF".
  status input   "? = help   q = quit   $ = EOL   ^ = BOL   > = EOF   < = BOF".

  return.

end.
