/*******************************************************************************
 *******************************************************************************
 **                                                                           **
 **                                                                           **
 **  Copyright 2003-2006 Tom Bascom, Greenfield Technologies                  **
 **  http://www.greenfieldtech.com                                            **
 **                                                                           **
 **  ProTop is free software; you can redistribute it and/or modify it        **
 **  under the terms of the GNU General Public License (GPL) as published     **
 **  by the Free Software Foundation; either version 2 of the License, or     **
 **  at your option) any later version.                                       **
 **                                                                           **
 **  ProTop is distributed in the hope that it will be useful, but WITHOUT    **
 **  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    **
 **  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License     **
 **  for more details.                                                        **
 **                                                                           **
 **  See TERMS.TXT for more information regarding the Terms and Conditions    **
 **  of use and alternative licensing options for this software.              **
 **                                                                           **
 **  A copy of the GPL is in GPL.TXT which was provided with this package.    **
 **                                                                           **
 **  See http://www.fsf.org for more information about the GPL.               **
 **                                                                           **
 **                                                                           **
 *******************************************************************************
 *******************************************************************************
 *
 * tt_lob.i
 *
 */

define temp-table tt_lob no-undo
  field xid      as integer
  field xvalid   as logical
  field LOBnum   as integer
  field areaNum  as integer
  field LOBPool  as character
  field LOBname  as character label "LOB Name" format "x(30)"
  field TblNm    as character label "Table Name" format "x(30)"
  field LOB-cre  as decimal extent 5 label "Create"  format ">>>>>>>>>>>>>>>>9"
  field LOB-rd   as decimal extent 5 label "Read"    format ">>>>>>>>>>>>>>>>9"
  field LOB-upd  as decimal extent 5 label "Update"  format ">>>>>>>>>>>>>>>>9"
  field LOB-del  as decimal extent 5 label "Delete"  format ">>>>>>>>>>>>>>>>9"
  field LOB-osrd as decimal extent 5 label "OS Read" format ">>>>>>>>>>>>>>>>9"
  field pct-cre  as decimal   label "%" format ">>9.99"
  field pct-rd   as decimal   label "%" format ">>9.99"
  field pct-upd  as decimal   label "%" format ">>9.99"
  field pct-del  as decimal   label "%" format ">>9.99"
  index xid-idx is unique primary xid.

procedure upd-tt_lob:

  define input parameter p_tbl as integer no-undo.

  define variable x as integer no-undo.
  define variable z as integer no-undo.

  define variable bh  as handle no-undo.
  define variable bf  as handle no-undo.

  do-SumSample( output x, output z ).

  find dictdb._LOBStat no-lock where _LOBStat-id = p_tbl no-error.

  find tt_lob no-lock where tt_lob.xid = _LOBStat-id no-error.
  if not available tt_lob then return.

  find tt_lob exclusive-lock where tt_lob.xid = p_tbl no-error.

  if available tt_lob then
    tt_lob.xvalid = yes.
   else
    do:
      create tt_lob.
      assign
        tt_lob.xid      = p_tbl
        tt_lob.xvalid   = yes
        tt_lob.lobName  = ( if available( _LOBStat ) then "" else "*" ) + tt_lob.lobname
        tt_lob.lobNum   = tt_lob.xid
        tt_lob.areanum  = tt_lob.areanum
        tt_lob.lobPool  = tt_lob.lobPool
      .

      find tt_tbl where tt_tbl.xid = _LOBStat._LOBStat-Table no-error.
      if available tt_tbl then
        tt_lob.tblNm = tt_tbl.tblname.

      if available _LOBStat then		/* _LOBStat range may not be active...	*/
        do:
          assign
            {lib/init-xrec.i tt_lob.lob-cre  _LOBStat-create}
            {lib/init-xrec.i tt_lob.lob-rd   _LOBStat-read}
            {lib/init-xrec.i tt_lob.lob-upd  _LOBStat-update}
            {lib/init-xrec.i tt_lob.lob-del  _LOBStat-delete}
            {lib/init-xrec.i tt_lob.lob-osrd _LOBStat-OSRead}
          .
        end.
    end.

  if available _LOBStat then		/* _LOBStat range may not be active...	*/
    do:

      assign
        {lib/upd-xrec.i tt_lob.lob-cre  _LOBStat-create}
        {lib/upd-xrec.i tt_lob.lob-rd   _LOBStat-read}
        {lib/upd-xrec.i tt_lob.lob-upd  _LOBStat-update}
        {lib/upd-xrec.i tt_lob.lob-del  _LOBStat-delete}
        {lib/upd-xrec.i tt_lob.lob-osrd _LOBStat-OSRead}
      .

    end.

  return.

end.

/* this will be called by the including procedure's mon-update procedure
 *
 */

procedure age_table:

  for each tt_lob exclusive-lock:

    if tt_lob.xvalid = no then
      delete tt_lob.
     else
      assign
        tt_lob.xvalid = no
      .

  end.

  return.

end.
